local _M = {}

function _M.WriteCastInfo()
  local bridge_tracer = require "opentracing_bridge_tracer"
  local GenericObjectPool = require "GenericObjectPool"
  local CastInfoServiceClient = require 'media_service_CastInfoService'
  local ngx = ngx
  local cjson = require("cjson")
  local service_config = cjson.decode(ngx.shared.config:get("service"))

  local req_id = tonumber(string.sub(ngx.var.request_id, 0, 15), 16)
  local tracer = bridge_tracer.new_from_global()
  local parent_span_context = tracer:binary_extract(ngx.var.opentracing_binary_context)
  local span = tracer:start_span("WriteCastInfo  ", {["references"] = {{"child_of", parent_span_context}}})
  local carrier = {}
  tracer:text_map_inject(span:context(), carrier)

  ngx.req.read_body()
  local data = ngx.req.get_body_data()

  if not data then
    ngx.status = ngx.HTTP_BAD_REQUEST
    ngx.say("Empty body")
    ngx.log(ngx.ERR, "Empty body")
    ngx.exit(ngx.HTTP_BAD_REQUEST)
  end

  local cast_info = cjson.decode(data)
  if (cast_info["cast_info_id"] == nil or cast_info["name"] == nil or
      cast_info["gender"] == nil or cast_info["intro"] == nil) then
    ngx.status = ngx.HTTP_BAD_REQUEST
    ngx.say("Incomplete arguments")
    ngx.log(ngx.ERR, "Incomplete arguments")
    ngx.exit(ngx.HTTP_BAD_REQUEST)
  end

  local client = GenericObjectPool:connection(
      CastInfoServiceClient, service_config["cast-info-service"]["addr"],
      service_config["cast-info-service"]["port"],
      service_config["cast-info-service"]["http_path"])
  client:WriteCastInfo(req_id, cast_info["cast_info_id"], cast_info["name"],
      cast_info["gender"], cast_info["intro"],  carrier)
  GenericObjectPool:returnConnection(client)

end

return _M